from flask import Flask
from flask import jsonify
from flask import abort, make_response, request, render_template

from pymongo import MongoClient
from bson import ObjectId
import json
import re

from app.shared import helper
from app.shared import db_init

from ipdb import set_trace
# import numpy as np
# from sklearn import neighbors, datasets

from app import app

baseURL = '/api/v1.0/'

@app.route(baseURL + 'exercises/<string:_id>', methods=['GET'])
def get_exercises_by_id(_id):
    try:
        exercises = db_init.db.exercises

        exercise = exercises.find_one({"_id" : ObjectId(_id)})

        if not exercise:
            abort(404)

        exercise['_id'] = str(exercise['_id'])
        return jsonify({'exercise' : exercise}), 200

    except Exception as inst:
        abort(503)


@app.route(baseURL + 'exercises', methods=['POST'])
def add_exercises():
    try:
        exercises = db_init.db.exercises
        package = request.get_json()

        if not package:
            abort(404)

        if exercises.find_one({"exercise_id" : package['exercise_id']}):
            return make_response(jsonify({'error': 'Mad request',
                                          'message': 'Exercise exists, choose another ID'}), 400)

        exercise = {
            'title' : package['title'],
            'exercise_id' : int(package['exercise_id']),
            'curriculum' : package['curriculum'],
            'text' : package['text'],
            'hint' : package['hint'],
            'result' : package['result'],
            'version' : 1,
            'created' : helper.utcnow(),
            'error_message' : package['error_message'],
            'timed' : package['timed'],
            'editor' : package['editor']
        }

        exercise_id = exercises.insert_one(exercise).inserted_id

        exercise['_id'] = str(exercise['_id'])
        return jsonify({'exercise' : exercise}), 200

    except Exception as inst:
        abort(503)

@app.route(baseURL + 'exercises/<string:_id>', methods=['PUT'])
def update_exercise(_id):
    try:
        exercises = db_init.db.exercises
        package = request.get_json()

        if not exercises.find_one({"_id" : ObjectId(_id)}):
            abort(404)

        exercise_version = exercises.find_one({"_id" : ObjectId(_id)})['version']

        exercise = {
            'title' : package['title'],
            'exercise_id' : int(package['exercise_id']),
            'curriculum' : package['curriculum'],
            'text' : package['text'],
            'hint' : package['hint'],
            'result' : package['result'],
            'version' : exercise_version + 1,
            'created' : helper.utcnow(),
            'error_message' : package['error_message'],
            'timed' : package['timed'],
            'editor' : package['editor']
        }

        exercise_id = exercises.insert_one(exercise).inserted_id

        exercise['_id'] = str(exercise['_id'])
        return jsonify({'exercise' : exercise}), 200

    except Exception as inst:
        abort(503)

# Get the next excercise for a user
@app.route(baseURL + 'users/exercise/<string:user_id>', methods=['GET'])
def get_exercise(user_id):
    try:
        exercises = db_init.db.exercises
        users = db_init.db.users

        last_exercise = exercises.find().sort('exercise_id', -1).limit(1)[0]['exercise_id']

        if not users.find_one({"_id" : ObjectId(user_id)}):
            abort(404)

        user = users.find_one({"_id" : ObjectId(user_id)})
        current_exercise = user['current_exercise']

        if not "experience" in user or user["experience"] == 0:
            user_status = False
        else:
            user_status = True

        # Finds random exercise with that exercise_id, not the last one
        sorted_exercises = exercises.find({"exercise_id" : current_exercise}).sort('version', 1)

        exercise = helper.unique_exercises(sorted_exercises)

        # There's probably a better way to do this
        sorted_exercises = exercises.find().sort([('exercise_id', 1), ('version', 1)])
        number_of_exercises = len(helper.unique_exercises(sorted_exercises))

        if not exercise:
            # TODO: User's current_exercise attribute should be changed
            # so that I don't have to add one to the last_exercise id
            # Hackety Hack McHack
            if current_exercise == last_exercise + 1:
                presentable_exercise = {
                    'title' : 'You did it!',
                    'exercise_id' : number_of_exercises,
                    'curriculum' : 'You have completed the tutorial. \nDo you want to try again? Just log the string "restart" to the console:) \nThank you!',
                    'text' : 'Log the string "restart" to the console to do the tutorial again.',
                    'hint' : '',
                    'error_message' : '',
                    'number_of_exercises' : number_of_exercises,
                    'user_status' : user_status,
                    'timed' : False,
                    'editor' : ""
                }

                return jsonify({'exercise' : presentable_exercise}), 200
            else:
                abort(404)

        presentable_exercise = {
            'title' : exercise[0]['title'],
            'exercise_id' : exercise[0]['exercise_id'],
            'curriculum' : exercise[0]['curriculum'],
            'text' : exercise[0]['text'],
            'hint' : exercise[0]['hint'],
            'error_message' : exercise[0]['error_message'],
            'number_of_exercises' : number_of_exercises,
            'user_status' : user_status,
            'timed' : exercise[0]['timed'],
            'editor' : exercise[0]['editor']
        }

        return jsonify({'exercise' : presentable_exercise}), 200

    except Exception as inst:
        print(type(inst))
        print('error', inst)
        abort(503)

# Get all sessions
# @app.route(baseURL + 'sessions', methods=['GET'])
# def get_sessions():
#     try:
#         sessions = db_init.db.sessions
#         response = []
#         for session in sessions.find({"user_id": int(2)}):
#             response.append({'user_id' : session['user_id']})
#
#         return jsonify({'response': response})
#
#     except Exception as inst:
#         abort(503)

# Get sessions for one user
# @app.route(baseURL + 'sessions/<string:user_id>', methods=['GET'])
# def get_sessions_by_id(user_id):
#     try:
#         sessions = db_init.db.sessions
#         response = []
#         for session in sessions.find({"user_id": user_id}):
#             response.append({'user_id' : session['user_id'],
#                              'timestamp' : session['timestamp'],
#                              'keylog' : session['keylog'],
#                              'emotion' : session['emotion'],
#                              'result' : session['result'],
#                              'number_of_misses' : session['number_of_misses'],
#                              'task_id' : session['task_id']})
#         if len(response) == 0:
#             abort(404)
#         return jsonify({'response' : response})
#
#     except Exception as inst:
#         abort(503)

# Post a session
@app.route(baseURL + 'sessions', methods=['POST'])
def add_session():
    try:
        package = request.get_json()
        cookie = request.cookies
        user_agent = request.user_agent

        if not package:
            abort(400)

        exercises = db_init.db.exercises
        sessions = db_init.db.sessions
        users = db_init.db.users

        # Finds the user's state
        user = users.find_one({"_id" : ObjectId(package['user_id'])})

        # Set the user's user agent if it is unknown
        if user['user_agent'] == 'unknown':
            users.update_one({
                "_id" : ObjectId(package['user_id'])
            },
            {
                "$set": {
                    "user_agent" : str(user_agent)
                }
            })

        # Find the correct exercise
        sorted_exercises = exercises.find({"exercise_id" : user['current_exercise']}).sort('version', 1)
        exercise = helper.unique_exercises(sorted_exercises)

        if exercise:
            correct_answer = exercise[0]['result']
            error_message = exercise[0]['error_message']
        else:
            correct_answer = "restart" # Need a correct answer that can handle any string
                                    # so that when there are no more exercises, the userId
                                    # can write anything and stil add a session on it
            error_message = 'There are no more exercises. Log "restart" to go again.'

        session = {
           'user_id' : cookie['adaptId'],
           'timestamp' : helper.utcnow(),
           'keylog' : package['keystrokes'],
           'emotion' : package['emotional_state'],
           'code' : package['code'],
           'result' : package['result'],
           'number_of_misses' : package['number_of_misses'],
           'exercise_id' : package['exercise_id'],
           'pulse' : package['pulse'],
           'min_pulse' : 0,
           'max_pulse' : 0,
           'was_reset' : package['was_reset'],
           'timer_affected' : package['timer_affected'],
           'timer_affected_reason' : package['timer_affected_reason'],
           'removal_affected' : package['removal_affected'],
           'removal_affected_reason' : package['removal_affected_reason']
        }

        session_id = sessions.insert_one(session).inserted_id

        actual_result = package['result'].lower().replace("\n", "").replace(" ", "").replace("\"", "'")
        expected_result = correct_answer.lower().replace("\n", "").replace(" ", "").replace("\"", "'")

        if actual_result == expected_result:
            result = {
                'result' : package['result'],
                'status' : 'success'
            }

            # There's probably a better way to do this
            number_of_exercises = len(helper.unique_exercises(exercises.find().sort([('exercise_id', 1), ('version', 1)])))

            if package['exercise_id'] == number_of_exercises:
                users.update_one({
                    "_id" : ObjectId(package['user_id'])
                },
                {
                    "$set": {
                        "current_exercise" : 1
                    }
                })
            # Update the user's last completed exercise
            # This is pretty naive. It has no idea what exercise
            # was completed. It is possible to solve the same exercise twice.
            # Doing so will yield the wrong answer the second time because the
            # counter is +1
            elif package['exercise_id'] == user['current_exercise']:
                users.update_one({
                    "_id" : ObjectId(package['user_id'])
                },
                {
                    "$set": {
                        "current_exercise" : user['current_exercise'] + 1
                    }
                })
        # Check if the first part of the result is 'error;'
        elif package['result'][0:6] == 'error;':
            result = {
                'status' : 'error',
                'result' : package['result'],
                'error_message' : error_message
            }
        else:
            result = {
                'status' : 'wrong answer',
                'result' : package['result'],
                'answer' : correct_answer,
                'error_message' : error_message
            }

        # Use this if the session dictionary should be returned
        # session['_id'] = str(session['_id'])
        # return jsonify({'session' : session}), 200
        return jsonify({'result' : result}), 200

    except Exception as inst:
        abort(503)

# Get a user
# @app.route(baseURL + 'users/<string:user_id>', methods=['GET'])
# def get_users_by_id(user_id):
#     users = db_init.db.users
#     user = users.find_one({"_id" : ObjectId(user_id)})
#
#     if not user:
#         abort(404)
#
#     user['_id'] = str(user['_id'])
#     return jsonify({'user' : user}), 200

# Post a user
@app.route(baseURL + 'users', methods=['POST'])
def post_users():
    try:
        users = db_init.db.users

        package = request.get_json()

        if not package:
            abort(400)

        user = helper.add_user()

        user['_id'] = str(user['_id'])
        return jsonify({'result' : user}), 200

    except Exception as inst:
        abort(503)

@app.route(baseURL + 'users/update', methods=['POST'])
def update_users():
    try:
        package = request.get_json()
        cookie = request.cookies
        users = db_init.db.users

        users.update_one({
            "_id" : ObjectId(cookie['adaptId'])
        },
        {
            "$set": {
                "gender" : package["gender"],
                "experience" : int(package["experience"]),
                "years_of_experience" : float(package["years_of_experience"]),
                "years_before_university" : float(package["years_before_university"]),
                "first_learned" : package["first_learned"],
                "languages" : package["languages"]
            }
        })

        return jsonify({'result' : 'success'}), 200

    except Exception as inst:
        abort(503)

# Post feedback
@app.route(baseURL + 'feedback', methods=['POST'])
def add_feedback():
    try:
        feedbacks = db_init.db.feedback

        package = request.get_json()

        if not package:
            abort(404)

        feedback = {
            'user_id' : package['user_id'],
            'exercise_id' : package['exercise_id'],
            'feedback_text' : package['feedback_text'],
            'created' : helper.utcnow(),
            'fixed' : False,
            'fixed_timestamp' : 0
        }


        feedback_id = feedbacks.insert_one(feedback).inserted_id
        feedback['_id'] = str(feedback['_id'])
        return jsonify({'result' : 'success'}), 200

    except Exception as inst:
        abort(503)

#Update a user's state
# @app.route(baseURL + 'users/<string:user_id>', methods=['PUT'])
# def update_users_by_id(user_id):
#     users = db_init.db.users
#     package = request.get_json()
#
#     if not package:
#         abort(404)
#
#     users.update_one({
#         "_id" : ObjectId(user_id)
#     },
#     {
#         "$set": {
#             "current_exercise" : package['current_exercise']
#         }
#     })
#
#     updated_user = users.find_one({"_id" : ObjectId(user_id)})
#     updated_user["_id"] = str(updated_user["_id"])
#     return jsonify({'user' : updated_user}), 200

@app.after_request
def add_header(response):
    return response

@app.errorhandler(404)
def not_found(error):
    return render_template('404.html'), 404

@app.errorhandler(400)
def bad_request(error):
    return make_response(jsonify({'error': 'Bad request'}), 400)

@app.errorhandler(503)
def service_unavailable(error):
    return make_response(jsonify({'error' : 'Systems offline'}), 503)
