"use strict";

const title = document.getElementById('exercise-title');
const exerciseId = document.getElementById('exercise-id');
const curriculum = document.getElementById('exercise-curriculum');
const text = document.getElementById('exercise-text');
const hint = document.getElementById('exercise-hint');
const result = document.getElementById('exercise-result');
const addExerciseButton = document.getElementById('add-exercise');
const errorMessage = document.getElementById('error-message');
const exerciseList = document.getElementById('exercise-list');
const adminTitle = document.getElementById('admin-title');
const updateExerciseButton = document.getElementById('update-exercise');
const exerciseUniqueId = document.getElementById('exercise-unique-id');
const version = document.getElementById('version');
const newExerciseButton = document.getElementById('new-exercise');
const deleteExerciseButton = document.getElementById('delete-exercise');
const viewExerciseTitle = document.getElementById('view-exercise-title');
const exerciseErrorMessage = document.getElementById('exercise-error-message');
const timed = document.getElementById('exercise-timed');
const editorContent = document.getElementById('exericse-editor-content');

addExerciseButton.addEventListener('click', addExerciseForm);
updateExerciseButton.addEventListener('click', updateExercise);
newExerciseButton.addEventListener('click', addExercise)

function addExercise(){
  clearForm('exercise-form')
  exerciseUniqueId.innerHTML = "";
  version.innerHTML = "";
  viewExerciseTitle.innerHTML = "Add new exercise";
  addExerciseButton.style.display = 'inline';
  updateExerciseButton.style.display = 'none';
  deleteExerciseButton.style.display = 'none';
}

function addExerciseForm(event){

  if(title.value && exerciseId.value && curriculum.value && text.value && hint.value && exerciseErrorMessage.value && result.value){
    event.preventDefault();

    if(parseInt(exerciseId.value) >= 0){
      const exercise = {
        'title' : title.value,
        'exercise_id' : parseInt(exerciseId.value),
        'curriculum' : curriculum.value,
        'text' : text.value,
        'hint' : hint.value,
        'error_message' : exerciseErrorMessage.value,
        'editor' : editorContent.value,
        'result' : result.value,
        'timed' : timed.checked
      };
      errorMessage.style.display = 'none';
      postExercise(displayExercises, displayErrorMessage, exercise);

    } else {
      displayErrorMessage('The ID field must be a positive number');
    }

  } else {
    displayErrorMessage('You have left some input fields blank');
  }

}

function viewExercise(_id){
  getExercise(displaySingleExercise, displayErrorMessage, _id);
}

function updateExercise(){
  // This is the same as addExerciseForm, and should be moved to it's own function
  // only exception is the _id attribute in exercise
  if(title.value && exerciseId.value && curriculum.value && text.value && hint.value && exerciseErrorMessage.value && result.value){
    event.preventDefault();

    if(parseInt(exerciseId.value) >= 0){
      const exercise = {
        '_id' : exerciseUniqueId.textContent,
        'title' : title.value,
        'exercise_id' : parseInt(exerciseId.value),
        'curriculum' : curriculum.value,
        'text' : text.value,
        'hint' : hint.value,
        'error_message' : exerciseErrorMessage.value,
        'editor' : editorContent.value,
        'result' : result.value,
        'timed' : timed.checked
      }

      errorMessage.style.display = 'none'
      putExercise(displaySingleExercise, displayErrorMessage, exercise)

      // Removes the old exercise element in exercises list
      document.getElementById(exerciseUniqueId.textContent).style.display = 'none';

      // Add the link to the updated exercise in exercises list
      displayExercises(JSON.stringify({'exercise': exercise}))

    } else {
      displayErrorMessage('The ID field must be greater than zero')
    }

  } else {
    displayErrorMessage('You have left some input fields blank')
  }
}

function displaySingleExercise(exercise){
  addExerciseButton.style.display = 'none'
  updateExerciseButton.style.display = 'inline'
  deleteExerciseButton.style.display = 'inline'

  exercise = JSON.parse(exercise)
  title.value = exercise.exercise.title;
  exerciseId.value = parseInt(exercise.exercise.exercise_id);
  curriculum.value = exercise.exercise.curriculum;
  text.value = exercise.exercise.text;
  hint.value = exercise.exercise.hint;
  exerciseErrorMessage.value = exercise.exercise.error_message;
  editorContent.value = exercise.exercise.editor;
  result.value = exercise.exercise.result;
  timed.checked = exercise.exercise.timed;
  exerciseUniqueId.innerHTML = exercise.exercise._id;
  version.innerHTML = ' v. ' + exercise.exercise.version;
  viewExerciseTitle.innerHTML = exercise.exercise.title;
}

function displayExercises(exercise){
  exercise = JSON.parse(exercise)
  const exerciseDiv = document.createElement('DIV')
  exerciseDiv.setAttribute('class', 'exercise')
  exerciseDiv.setAttribute('onclick', "viewExercise('" + exercise.exercise._id +"')")
  const textNode = document.createTextNode(exercise.exercise.exercise_id + '. ' + exercise.exercise.title)
  exerciseDiv.appendChild(textNode)

  exerciseList.appendChild(exerciseDiv)
  clearForm('exercise-form')
}

function displayErrorMessage(message){
  errorMessage.style.display = 'block'
  errorMessage.innerHTML = message
}

function getExercise(successCallback, errorCallback, id){
  return doGet('exercises/' + id, successCallback, errorCallback);
}

function postExercise(successCallback, errorCallback, data){
  return doPost('exercises', data, successCallback, errorCallback);
}

function putExercise(successCallback, errorCallback, data){
  return doPut('exercises/' + data._id, data, successCallback, errorCallback)
}
