from flask import render_template, make_response
from flask import request, abort
from app import app
from app.shared import helper
from app.shared import db_init
from datetime import datetime, timedelta
from bson import ObjectId

@app.route('/')
@app.route('/index')
def index():
    return render_template('index.html')

@app.route('/learn')
def learn():
    try:
        exercises = db_init.db.exercises
        users = db_init.db.users

        # There's probably a better way to do this
        sorted_exercises = exercises.find().sort([('exercise_id', 1), ('version', 1)])
        number_of_exercises = len(helper.unique_exercises(sorted_exercises))

        # If number_of_exercises == 0:
        # return template med det finnes ingen exercises

        if 'adaptId' in request.cookies:
            user_id = request.cookies['adaptId']

            # In case of the database being dropped, and the
            # user still has a cookie locally, this will
            # add the user's id, so that the user won't have
            # to delete his cookie
            if not users.find_one({"_id" : ObjectId(user_id)}):
                if len(user_id) == 24:

                    # This is identical to the add_user helper method
                    # except it has the _id attribute
                    user = {
                        '_id' : ObjectId(user_id),
                        'username' : 'nousername',
                        'user_agent' : 'unknown',
                        'current_exercise' : 0,
                        'completed' : False,
                        'created' : helper.utcnow(),
                        'password' : 'doesntexist',
                        'email' : 'no@email.com',
                        'is_admin' : False
                    }

                    users.insert_one(user)

            current_exercise = users.find_one({"_id" : ObjectId(user_id)})['current_exercise']

            last_exercise = exercises.find().sort('exercise_id', -1).limit(1)[0]['exercise_id']

            if current_exercise <= last_exercise:
                sorted_exercises = exercises.find({"exercise_id" : current_exercise}).sort('version', 1)
                exercise = helper.unique_exercises(sorted_exercises)

                presentable_exercise = {
                    'title' : exercise[0]['title'],
                    'exercise_id' : exercise[0]['exercise_id'],
                    'text' : exercise[0]['text'],
                    'curriculum' : exercise[0]['curriculum'],
                    'hint' : exercise[0]['hint'],
                    'error_message' : exercise[0]['error_message'],
                    'number_of_exercises' : number_of_exercises,
                    'timed' : int(exercise[0]['timed']),
                    'editor' : exercise[0]['editor']
                }

                return render_template('learn.html',
                                        exercise = presentable_exercise)
            else:
                presentable_exercise = {
                    'title' : 'You did it!',
                    'curriculum' : 'You have completed the tutorial. \nDo you want to try again? Just log the string "restart" to the console:) \nThank you!',
                    'text' : 'Log the string "restart" to the console to do the tutorial again.',
                    'hint' : "This tutorial didn't cover objects, so you should read about them.",
                    'error_message' : 'You have completed the tutorial, but you can start again.',
                    'exercise_id' : number_of_exercises,
                    'number_of_exercises' : number_of_exercises,
                    'timed' : False,
                    'editor' : ""
                }

                return render_template('learn.html',
                                       exercise = presentable_exercise)
        else:
            expire_datetime = datetime.utcnow() + timedelta(days=365)
            user = helper.add_user()
            user['_id'] = str(user['_id'])

            # Starts with the first exercise
            # TODO: choose exercise 1 with highest version number
            sorted_exercises = exercises.find({"exercise_id" : 0}).sort('version', 1)
            exercise = helper.unique_exercises(sorted_exercises)

            presentable_exercise = {
                'title' : exercise[0]['title'],
                'exercise_id' : exercise[0]['exercise_id'],
                'text' : exercise[0]['text'],
                'curriculum' : exercise[0]['curriculum'],
                'hint' : exercise[0]['hint'],
                'error_message' : exercise[0]['error_message'],
                'number_of_exercises' : number_of_exercises,
                'timed' : int(exercise[0]['timed']),
                'editor' : exercise[0]['editor']
            }

            resp = make_response(render_template('learn.html',
                                                 user = user['_id'],
                                                 exercise = presentable_exercise))
            resp.set_cookie('adaptId', user['_id'], expires=expire_datetime)
            return resp

    except Exception:

        error = {
            'title' : 'Error',
            'exercise_id' : 0,
            'curriculum' : "Adapt couldn't connect to the database. \n In the meantime I would recommend that you read Marijn Haverbeke's Eloquent JavaScript at http://eloquentjavascript.net.",
            'text' : "",
            'hint' : "Take a break and look outside:)",
            'number_of_exercises' : 0,
        }

        return render_template('learn.html',
                               exercise = error)

@app.route('/admin')
def admin():
    try:
        exercises = db_init.db.exercises
        users = db_init.db.users

        if 'adaptId' in request.cookies:
            user_id = request.cookies['adaptId']

            # TODO: Make a better way to restrict access
            if users.find_one({"_id" : ObjectId(user_id)})['is_admin']:
                sorted_exercises = exercises.find().sort([('exercise_id', 1), ('version', 1)])

                resp = helper.unique_exercises(sorted_exercises)

                return make_response(render_template('admin.html',
                                                     user = user_id,
                                                     exercises = resp))

            else:
                return render_template('404.html'), 404
        else:
            return render_template('404.html'), 404

    except Exception:
        return render_template('404.html'), 404

# http://scikit-learn.org/stable/learn.html
# https://www.dataquest.io/blog/k-nearest-neighbors-in-python/
# http://machinelearningmastery.com/get-your-hands-dirty-with-scikit-learn-now/
