"use strict";

/* 00. TABLE OF CONTENT
01. Elements from DOM
02. Event listeners
03.
04. Object constructors
05. Keylogger
06.
07. Database
08.
09.

Project: Detect emotions using keystroke dynamics while coding
Written by: Thor Håkon Bredesen
Year: 2017
*/

// 01. ELEMENTS FROM DOM
const aceEditor = document.getElementById('ace-editor');
const runButton = document.getElementById('run');
const nextButton = document.getElementById('next');
const editorConsole = document.getElementById('editor-console');

const emotionModal = document.getElementById('emotion-modal');
const userModal = document.getElementById('user-modal');
const registerEmotion = document.getElementById('register-emotion');
const updateUserButton = document.getElementById('update-user');
const closeModalButtons = document.getElementsByClassName('close-modal');
const emotionFormError = document.getElementById('emotion-form-error');

const titleText = document.getElementById('display-title')
const curriculumText = document.getElementById('display-curriculum');
const exerciseText = document.getElementById('display-text');
const hintText = document.getElementById('display-hint');
const panelTitle = document.getElementsByClassName('panel-title');

const username = document.getElementById('username')
const exerciseIdListElement = document.getElementById('exercise-id')

const editorMessageWrapper = document.getElementById('editor-message-wrapper')
const editorMessage = document.getElementById('editor-message')
const exitMessage = document.getElementById('exit-message')

const progressExerciseNumber = document.getElementById('progress-exercise-number');
const progressExerciseTotal = document.getElementById('progress-exercise-total');

const SendFeedbackButton = document.getElementById('send-feedback-button');
const feedbackText = document.getElementById('feedback-text');
const feedbackRespone = document.getElementById('feedback-response');

const noLanguageCheckbox = document.getElementById('no-language');
const yearsOfExperienceInput = document.getElementById('years_of_experience');
const yearsBeforeUniversityInput = document.getElementById('years_before_university');
const firstLearned = document.getElementById('first-learned');
const otherLanguagesInput = document.getElementById('other-lang');
const freetextLang = document.getElementById('freetext-lang');
const languages = document.getElementsByName('languages');

const clock = document.getElementById('clock');
const exerciseTimedListElement = document.getElementById('exercise-timed');

const timerAffected = document.getElementById('timer-affected');
const timerAffectedYes = document.getElementById('timer-affected-yes');
const timerAffectedNo = document.getElementById('timer-affected-no');
const timerAffectedReasonTitle = document.getElementById('timer-affected-reason-title');
const timerAffectedReason = document.getElementById('timer-affected-reason');

const removalAffected = document.getElementById('removal-affected');
const removalAffectedYes = document.getElementById('removal-affected-yes');
const removalAffectedNo = document.getElementById('removal-affected-no');
const removalAffectedReasonTitle = document.getElementById('removal-affected-reason-title');
const removalAffectedReason = document.getElementById('removal-affected-reason');

nextButton.disabled = true;

// 02. EVENT LISTENERS
aceEditor.addEventListener("keydown", keyboardLogger);
aceEditor.addEventListener("keyup", keyboardLogger);
runButton.addEventListener('click', run);
registerEmotion.addEventListener('click', RegisterForm);
nextButton.addEventListener('click', getNextExercise);
exitMessage.addEventListener('click', closeEditorMessage);
SendFeedbackButton.addEventListener('click', sendFeedback);
updateUserButton.addEventListener('click', updateUser);
yearsBeforeUniversityInput.addEventListener('change', updateFirstLearnedSelect);
noLanguageCheckbox.addEventListener('change', updateYearsOfExperience);
timerAffectedYes.addEventListener('change', timerAffectForm);
timerAffectedNo.addEventListener('change', timerAffectForm);
removalAffectedYes.addEventListener('change', removalAffectForm);
removalAffectedNo.addEventListener('change', removalAffectForm);


for (var a = 0; a < closeModalButtons.length; a++) {
  closeModalButtons[a].addEventListener('click', closeModal, false)
}
for (var b = 0; b < panelTitle.length; b++) {
  panelTitle[b].addEventListener('click', changePanelState, false)
}
for (var c = 0; c < languages.length; c++) {
  languages[c].addEventListener('change', toggleLanguageCheckboxes, false)
}

const sessions = [];
const feedbacks = [];
var wasReset = false;
var timeInterval = null;

// 04. OBJECT CONSTRUCTORS

function Keystroke(key_code, time_down, time_up) {
    this.key_code = key_code;
    this.time_down = time_down;
    this.time_up = time_up;
}

function Session(user_id, emotion, keystrokes, code, result, number_of_misses,
                 exercise_id, pulse, was_reset, timer_affected,
                 timer_affected_reason, removal_affected, removal_affected_reason) {

  this.user_id = user_id;
  this.emotional_state = emotion;
  this.keystrokes = keystrokes;
  this.code = code;
  this.result = result;
  this.number_of_misses = number_of_misses;
  this.exercise_id = exercise_id
  this.pulse = pulse;
  this.was_reset = was_reset;
  this.timer_affected = timer_affected;
  this.timer_affected_reason = timer_affected_reason;
  this.removal_affected = removal_affected;
  this.removal_affected_reason = removal_affected_reason;
}

function Feedback(user_id, exercise_id, feedback_text){
  this.user_id = user_id;
  this.exercise_id = exercise_id;
  this.feedback_text = feedback_text;
}

function addSession(result, code) {
  var exerciseId = undefined
  if (typeof(Storage) !== "undefined" && window.localStorage.getItem('exerciseId')){
    exerciseId = JSON.parse(window.localStorage.getItem('exerciseId'))
  } else {
    exerciseId = getCookie("exerciseId")
  }

  if(exerciseId >= 0){
    sessions[sessions.length] = new Session(getCookie("adaptId"),            // user_id
                                            undefined,                       // emotion
                                            keylog,                          // keylog
                                            code,                            // code
                                            result,                          // result
                                            calculateNumberOfMisses(keylog), // number_of_misses
                                            exerciseId,                      // exercise_id
                                            0,                               // pulse
                                            wasReset,                        // was_reset
                                            undefined,                       // timer_affected
                                            undefined,                       // timer_affected_reason
                                            undefined,                       // removal_affected
                                            undefined);                      // removal_affected_reason
  }
}

function addFeedback(feedbackText){
  var exerciseId = undefined
  if (typeof(Storage) !== "undefined" && window.localStorage.getItem('exerciseId')){
    exerciseId = JSON.parse(window.localStorage.getItem('exerciseId'))
  } else {
    exerciseId = getCookie("exerciseId")
  }

  if(exerciseId >= 0) {
    feedbacks[feedbacks.length] = new Feedback(getCookie("adaptId"),  // user_id
                                               exerciseId,            // exercise_id
                                               feedbackText)          // feedback text
  }
}

// 05. KEYLOGGER FUNCTIONS

// Keylog datastructure: list with objects
const keylog = [];

function keyboardLogger(event) {

  const randum = Math.floor(Math.random() * 600) + 1
  var recordEvent = true;
  var d = keylog.length

  // Reset the editor ish every 1000 key event
  if(randum == 42){
    editor.setValue("");
    wasReset = true
    keylog[d] = new Keystroke(256, event.timeStamp, event.timeStamp)
  }

  if(editor.getValue() && keylog.length > 0) {
    runButton.disabled = false;
  } else {
    runButton.disabled = true;
  }

  // If the key is backspace, arrow left/up/right/down, delete
  if(event.keyCode == 8 || event.keyCode == 37 || event.keyCode == 38 ||
    event.keyCode == 39 || event.keyCode == 40 || event.keyCode == 46) {

    // These keys doesn't have a keydown event,
    // thus the dwell time can't be calculated
    keylog[d] = new Keystroke(event.keyCode, event.timeStamp, event.timeStamp);
  }

  // If the key is pressed down
  if (event.type == 'keydown') {

    // If the key is Cmd
    if (event.keyCode == 91) {
      recordEvent = false;
    }

    // If the keylog list is not empty
    else if (d > 0) {
      // Removes keydown events from the same key being held down
      if (keylog[d - 1].time_up == 0 && keylog[d - 1].key_code == event.keyCode) {
        recordEvent = false;
      }
    }

    // If the event should be recorded a new Keystroke object is created
    if (recordEvent) {
      keylog[d] = new Keystroke(event.keyCode, event.timeStamp, 0 );
    }
  }

  // If the key is released
  if (event.type == 'keyup') {

    // Loops through the recorded events, starting on the last
    for(var e = keylog.length - 1; e >= 0; e--) {

      // Finding the corrensponding keydown event
      if (keylog[e].key_code == event.keyCode && keylog[e].time_up == 0) {
        keylog[e].time_up = event.timeStamp;
        break;
      }
    }
  }
}

// RUN CODE FROM EDITOR

function run() {
  // Gets the code from the editor and store it in a string
  const code = editor.getValue();
  editorConsole.innerHTML = '';

  if(code) {
    closeEditorMessage();

    // Initializes a worker
    var worker = new Worker('static/js/consoleworker.js');

    // If the code executes
    worker.addEventListener('message', function(e) {
      var result = e.data;
      editorConsole.innerHTML = " ";

      if(wasReset){
        removalAffected.style.display = "block";
      } else {
        removalAffected.style.display = "none";
      }

      emotionModal.style.display = "block";
      addSession(result, code);
    });

    // If the code did not execute
    worker.addEventListener('error', function(e) {

      if(wasReset){
        removalAffected.style.display = "block";
      } else {
        removalAffected.style.display = "none";
      }

      emotionModal.style.display = "block";
      const errorMessage = 'error;' + e.message + ';' + e.lineno + ';' + e.colno
      addSession(errorMessage, code);
    });

    // Sends the code to the worker
    worker.postMessage(code);

    // If the returning code doesn't end, it will be terminated
    // TODO: Also add session when it time out

    setTimeout(function() {
      if (!editorConsole.innerHTML){
        worker.terminate();
        worker = null;
        addSession('timeout', code);
        editorConsole.innerHTML = "Your code timed out. \nDon't worry though, I stopped it for you.";
        editorMessage.innerHTML = "Your code got trapped in an infinte loop. Do some changes and try again:)";
        editorMessageWrapper.style.display = 'block';
      }
    }, 20000);

  }
  else{
    const message = "You must write something before you can run the code";
    displayEditorMessage(message);
  }
}

// Handles actions based on the result
function displayResult(code) {
  code = JSON.parse(code);

  if(code.result.status == 'success') {
    editorConsole.innerHTML = code.result.result;
    nextButton.disabled = false;
    runButton.disabled = true;
    clearInterval(timeInterval)
  }
  else if(code.result.status == 'wrong answer') {
    editorConsole.innerHTML = code.result.result;
    displayEditorMessage(code.result.error_message);
  }
  else if(code.result.status == 'error') {
    const errorString = code.result.result.split(';');
    editorConsole.innerHTML = errorString[1] + '\nat line ' + errorString[2] + ' and column ' + errorString[3];
    displayEditorMessage("You have an error in your code");
  }
  else {
    editorConsole.innerHTML = code.result.result;
    displayEditorMessage('Something went wrong on the server');
    // TODO: Add posibility to report a bug here
  }
}

// ERROR MESSAGE

// Display the editor message element
function displayEditorMessage(message){
  editorMessage.innerHTML = message
  editorMessageWrapper.style.display = 'block';
}

// Close the editor message element
function closeEditorMessage(){
  editorMessageWrapper.style.display = "none";
}

// 08. FORMS

function getRadioButtonValue(formIndex, radioName) {
  return document.forms[formIndex].elements[radioName].value
}

function RegisterForm(event) {
  if(getRadioButtonValue(0, 'emotion')) {
    event.preventDefault();
    var canSendData = true
    const emotion = getRadioButtonValue(0, 'emotion');

    if(timerAffected.style.display == "block"){
      var timerAffectedStatus = getRadioButtonValue(0, 'timer-affected');
      var timerReason = timerAffectedReason.value;
    } else {
      var timerAffectedStatus = 0;
      var timerReason = "";
    }

    if(removalAffected.style.display == "block"){
      var removalAffectedStatus = getRadioButtonValue(0, 'removal-affected');
      var removalReason = removalAffectedReason.value;
    } else {
      var removalAffectedStatus = 0;
      var removalReason = "";
    }

    sessions[sessions.length-1].emotional_state = emotion;
    sessions[sessions.length-1].timer_affected = parseInt(timerAffectedStatus);
    sessions[sessions.length-1].timer_affected_reason = timerReason;
    sessions[sessions.length-1].removal_affected = parseInt(removalAffectedStatus);
    sessions[sessions.length-1].removal_affected_reason = removalReason;

    clearForm('emotion-form');
    for (var key in sessions[sessions.length-1]) {
      if(sessions[sessions.length-1][key] == undefined) {
        canSendData = false;
        emotionFormError.style.display = "block";

        // TODO: add information on what's missing
        console.log("Can't send data, missing information")
        console.log(sessions[sessions.length-1])
        break;
      }
    }

    if(canSendData) {
      closeModal();
      postSessions(displayResult, error, sessions[sessions.length-1]);
      wasReset = false;
      keylog.length = 0;
      timerAffectedReasonTitle.style.display = "none";
      timerAffectedReason.style.display = "none";
      removalAffectedReasonTitle.style.display = "none";
      removalAffectedReason.style.display = "none";
    }
  }
}

function sendFeedback(event){
  if(feedbackText.value) {
    event.preventDefault();

    addFeedback(feedbackText.value)

    if(feedbacks[feedbacks.length-1]){
      postFeedback(displayFeedbackResult, error, feedbacks[feedbacks.length-1])
    }
  }
}

// CALCULATE NUMBER OF MISSESS (DELTE/BACKSPACE) IN KEYLOG
function calculateNumberOfMisses(keylog) {
  var misses = 0;

  for (var k = 0; k < keylog.length; k++) {
    // key_code 8 = backspace, key_code 46 = delete
    if(keylog[k].key_code == 8 || keylog[k].key_code == 46) {
      misses += 1;
    }
  }
  return misses;
}

function displayFeedbackResult(){
  clearForm('feedback-form')
  feedbackRespone.innerHTML = 'Thank you:)'
}

function updateUser(event){

  if(getRadioButtonValue(1, 'gender') &&
     getRadioButtonValue(1, 'experience-likert') &&
     yearsOfExperienceInput.value &&
     yearsBeforeUniversityInput.value &&
     firstLearned.value) {

    event.preventDefault();

    var canSendData = false;

    for (var f = 0; f < languages.length; f++) {
      if(languages[f].checked){
        canSendData = true;
        break;
      }
    }

    if(languages[languages.length-2].checked && !languages[languages.length-1].value){
      canSendData = false;
    }

    if(canSendData){
      const userInformation = {
        "gender" : getRadioButtonValue(1, 'gender'),
        "experience" : getRadioButtonValue(1, 'experience-likert'),
        "years_of_experience" : yearsOfExperienceInput.value,
        "years_before_university" : yearsBeforeUniversityInput.value,
        "first_learned" : firstLearned.value,
        "languages" : {}
      }

      // All predefined languages, except the last checkbox "other-lang"
      for (var g = 1; g < languages.length-2; g++) {
        if(languages[g].checked){
          userInformation["languages"][languages[g].value] = 2;
        } else {
          userInformation["languages"][languages[g].value] = 1;
        }
      }

      if(languages[languages.length-1].value){
        userInformation["languages"]["other"] = languages[languages.length-1].value;
      }

      postUpdateUser(success, error, userInformation)
      closeModal()
      clearForm('user-form')
    }

  }
}

function toggleLanguageCheckboxes(){

  if(languages[0].checked){
    for (var h = 1; h < languages.length; h++) {
      languages[h].disabled = true;
    }
  } else {
    for (var i = 1; i < languages.length; i++) {
      languages[i].disabled = false;
    }
  }

  // All predefined languages, except the last "freetext-lang"
  for (var j = 1; j < languages.length-1; j++) {
    if(languages[j].checked){
      languages[0].disabled = true;
      break;
    } else {
      languages[0].disabled = false;
    }
  }

  // If "other-lang" is checked, the text input is made visible
  if(languages[languages.length-2].checked){
    freetextLang.style.display = "block";
    freetextLang.setAttribute("required", true)
  } else {
    freetextLang.style.display = "none";
    freetextLang.setAttribute("required", false)
  }

}

function updateFirstLearnedSelect(event){
  if(yearsBeforeUniversityInput.value == 0){
    firstLearned.value = "university"
  } else {
    firstLearned.value = ""
  }
}

function updateYearsOfExperience(){
  if (noLanguageCheckbox.checked){
    yearsBeforeUniversityInput.value = 0;
    yearsOfExperienceInput.value = 0;
    firstLearned.value = "first_time";
  } else {
    yearsBeforeUniversityInput.value = "";
    yearsOfExperienceInput.value = "";
    firstLearned.value = "";
  }
}

function changePanelState(e){
  this.classList.toggle('active');

  const panel = this.nextElementSibling;
  if (panel.style.display === "block"){
    panel.style.display = "none";
  } else {
    panel.style.display = "block";
  }
}

function timerAffectForm(event){
  if(event.target.id == "timer-affected-yes"){
    timerAffectedReasonTitle.innerHTML = "How did the timer affect you?";
    timerAffectedReasonTitle.style.display = "block";
    timerAffectedReason.style.display = "block";

  } else if (event.target.id == "timer-affected-no") {
    timerAffectedReasonTitle.innerHTML = "Why didn't the timer affect you?";
    timerAffectedReasonTitle.style.display = "block";
    timerAffectedReason.style.display = "block";
  }
}

function removalAffectForm(event){
  if(event.target.id == "removal-affected-yes"){
    removalAffectedReasonTitle.innerHTML = "How did the deletion of text affect you?";
    removalAffectedReasonTitle.style.display = "block";
    removalAffectedReason.style.display = "block";

  } else if (event.target.id == "removal-affected-no") {
    removalAffectedReasonTitle.innerHTML = "Why didn't the deletion of text affect you?";
    removalAffectedReasonTitle.style.display = "block";
    removalAffectedReason.style.display = "block";
  }
}

// EXERCISE HANDLING

function displayExercise(exercise) {
  // reset the editor

  // Sets one space in the console, so that
  // the run function doesn't time out
  editorConsole.innerHTML = " ";
  feedbackRespone.innerHTML = "";

  exercise = JSON.parse(exercise);

  if(!exercise.exercise.user_status){
    userModal.style.display = "block";
  }

  if (typeof(Storage) !== "undefined"){
    window.localStorage.setItem('exerciseId', JSON.stringify(exercise.exercise.exercise_id));
  } else {
    const date = new Date();
    // Expires in 365 days
    date.setTime(date.getTime() + (365*24*60*60*1000));
    const expires = "expires=" + date.toUTCString();
    setCookie('exerciseId', JSON.stringify(exercise.exercise.exercise_id), expires)
  }

  runButton.disabled = true
  editor.setValue(exercise.exercise.editor);
  curriculumText.innerHTML = SplitTextOnLinebreak(exercise.exercise.curriculum);
  hintText.innerHTML = SplitTextOnLinebreak(exercise.exercise.hint);
  exerciseText.innerHTML = SplitTextOnLinebreak(exercise.exercise.text);
  titleText.innerHTML = exercise.exercise.title;
  nextButton.disabled = true;
  progressExerciseTotal.innerHTML = exercise.exercise.number_of_exercises;
  progressExerciseNumber.innerHTML = exercise.exercise.exercise_id;

  setTimer(exercise.exercise.timed)
}

function getNextExercise(){
  closeEditorMessage()
  getExercises(displayExercise, error, getCookie("adaptId"))
}

function setexerciseId(){
  if (typeof(Storage) !== "undefined"){
    window.localStorage.setItem('exerciseId', JSON.stringify(exerciseIdListElement.value));
  } else {
    const date = new Date();
    // Expires in 365 days
    date.setTime(date.getTime() + (365*24*60*60*1000));
    const expires = "expires=" + date.toUTCString();
    setCookie('exerciseId', JSON.stringify(exerciseIdListElement.values), expires)
  }
}

// TIMER

function setTimer(timerBoolean){
  if (timerBoolean){
    clock.style.display = "block";
    clock.style.color = "#F8F8F8";
    aceEditor.style.height = 'calc(100% - 90px)';
    timerAffected.style.display = "block";

    // Set countdown to four minutes
    timer(new Date(Date.parse(new Date()) + 4 * 60 * 1000));
  } else {
    clock.style.display = "none";
    aceEditor.style.height = 'calc(100% - 40px)';
    timerAffected.style.display = "none";
  }
}

function timer(deadline){
  const timerMinutes = document.getElementById('minutes');
  const timerSeconds = document.getElementById('seconds');
  const timerMilliseconds = document.getElementById('milliseconds');

  function updateTimer(){
    const t = getRemainingTime(deadline);

    timerMinutes.innerHTML = ('0' + t.minutes).slice(-2);
    timerSeconds.innerHTML = ('0' + t.seconds).slice(-2);
    // If less than one minute
    if(t.total < 60000){
      clock.style.color = "#EE7811";
    }

    if(t.total <= 0) {
      clock.style.color = "red";
      clearInterval(timeInterval)
    }
  }

  updateTimer()
  timeInterval = setInterval(updateTimer, 1000)
}

function getRemainingTime(endtime){
  const t = Date.parse(endtime) - Date.parse(new Date());
  const seconds = Math.floor( (t / 1000) % 60);
  const minutes = Math.floor( (t / (1000*60)) % 24);
  return {
    'total' : t,
    'minutes' : minutes,
    'seconds' : seconds,
  };
}

// COOKIE HANDLING

function setCookie(cookieName, cookieValue, expires) {
  document.cookie = cookieName + "=" + cookieValue + ";" + expires + ";path=/"
}

// Gets the value from the cookie
function getCookie(cookieName) {
  const name = cookieName + "=";
  const decodedCookie = decodeURIComponent(document.cookie);
  const cookieAttribute = decodedCookie.split(";");
  for (var l = 0; l < cookieAttribute.length; l++) {
    var cookie = cookieAttribute[l]
    while (cookie.charAt(0) == ' ') {
      cookie = cookie.substring(1);
    }
    if (cookie.indexOf(name) == 0) {
      return cookie.substring(name.length, cookie.length)
    }
  }
  return "";
}

// API REQUESTS

// Gets the next exercise for that user
function getExercises(successCallback, errorCallback, userId){
  return doGet('users/exercise/' + userId, successCallback, errorCallback);
}

// Posts a session to the database
function postSessions(successCallback, errorCallback, data){
  return doPost('sessions', data, successCallback, errorCallback);
}

function postFeedback(successCallback, errorCallback, data){
  return doPost('feedback', data, successCallback, errorCallback)
}

function postUpdateUser(successCallback, errorCallback, data){
  return doPost('users/update', data, successCallback, errorCallback)
}

// RUN APPLICATION
setexerciseId();
setTimer(exerciseTimedListElement.value);
